@@ -3,7 +3,14 @@ from datetime import datetime
 from enum import EnumMeta, Enum
 from typing import Type, List
 
-from pydictable.core import Field, DictAble
+from pydictable.type import Field, _BaseDictAble
+
+
+class DataValidationError(Exception):
+    def __init__(self, path: str, err):
+        super(DataValidationError, self).__init__(path, err)
+        self.path = path
+        self.err = err
 
 
 class StrField(Field):
@@ -77,7 +84,7 @@ class DatetimeField(Field):
 
 
 class ObjectField(Field):
-    def __init__(self, obj_type: Type[DictAble], required: bool=False):
+    def __init__(self, obj_type: Type[_BaseDictAble], required: bool=False):
         super(ObjectField, self).__init__(required=required)
         self.obj_type = obj_type
 
@@ -88,10 +95,14 @@ class ObjectField(Field):
         return None if v is None else v.to_dict()
 
     def validate_dict(self, field_name: str, v):
+        assert not self.required or v is not None
         self.obj_type(dict=v)
 
     def validate(self, field_name: str, v):
-        assert isinstance(v, DictAble)
+        assert isinstance(v, _BaseDictAble)
+
+    def of(self):
+        return self.obj_type.get_input_spec()
 
 
 class ListField(Field):
@@ -107,12 +118,19 @@ class ListField(Field):
 
     def validate_dict(self, field_name: str, v):
         assert type(v) == list
-        [self.obj_type.validate_dict(field_name, x) for x in v]
+        for i, _val in enumerate(v):
+            try:
+                self.obj_type.validate_dict(field_name, _val)
+            except AssertionError as e:
+                raise DataValidationError(f'[{i}]', str(e))
 
     def validate(self, field_name: str, v):
         assert type(v) == list
         [self.obj_type.validate(field_name, x) for x in v]
 
+    def of(self):
+        return self.obj_type.__class__.__name__
+
 
 class CustomField(Field, ABC):
     """
@@ -133,7 +151,7 @@ class CustomField(Field, ABC):
 class MultiTypeField(CustomField):
     TYPE_KEY = '__type'
 
-    def __init__(self, types: List[Type[DictAble]], *args, **kwargs):
+    def __init__(self, types: List[Type[_BaseDictAble]], *args, **kwargs):
         self.types_dict = {t.__name__: t for t in types}
         super(MultiTypeField, self).__init__(
             lambda d: self.types_dict[d[self.TYPE_KEY]](dict=d),
@@ -168,6 +186,9 @@ class EnumField(Field):
     def validate(self, field_name: str, v):
         assert isinstance(v, Enum)
 
+    def of(self):
+        return [e.value for e in self.enum]
+
 
 class DictField(Field):
     def from_dict(self, v):
@@ -184,7 +205,7 @@ class DictField(Field):
 
 
 class DictValueField(Field):
-    def __init__(self, value_type: Type[DictAble], required: bool = False, key: str = None):
+    def __init__(self, value_type: Type[_BaseDictAble], required: bool = False, key: str = None):
         self.value_type = value_type
         super(DictValueField, self).__init__(required, key)
 
@@ -201,4 +222,63 @@ class DictValueField(Field):
     def validate(self, field_name: str, v):
         assert type(v) == dict
         for val in v.values():
-            assert isinstance(val, DictAble)
+            assert isinstance(val, _BaseDictAble)
+
+
+class UnionField(Field):
+    def __init__(self, fields: List[Field], *args, **kwargs):
+        super(UnionField, self).__init__(*args, **kwargs)
+        self.fields_dict = {f.__class__.__name__: f for f in fields}
+
+    def from_dict(self, v):
+        for name, field in self.fields_dict.items():
+            try:
+                field.validate_dict('', v)
+                return field.from_dict(v)
+            except AssertionError:
+                pass
+        raise NotImplementedError()
+
+    def to_dict(self, v):
+        for name, field in self.fields_dict.items():
+            try:
+                field.validate('', v)
+                return field.to_dict(v)
+            except AssertionError:
+                pass
+        raise NotImplementedError()
+
+    def validate_dict(self, field_name: str, v):
+        for name, field in self.fields_dict.items():
+            try:
+                field.validate_dict('', v)
+                return
+            except AssertionError:
+                pass
+        raise AssertionError(f'{v} does not match for any of {list(self.fields_dict.keys())}')
+
+    def validate(self, field_name: str, v):
+        for name, field in self.fields_dict.items():
+            try:
+                field.validate('', v)
+                return
+            except AssertionError:
+                pass
+        raise AssertionError(f'{v} does not match for any of {list(self.fields_dict.keys())}')
+
+    def of(self):
+        return [f for f in self.fields_dict.keys()]
+
+
+class NoneField(Field):
+    def from_dict(self, v):
+        return None
+
+    def to_dict(self, v):
+        return None
+
+    def validate_dict(self, field_name: str, v):
+        assert v is None
+
+    def validate(self, field_name: str, v):
+        assert v is None
